import datetime
import re

from flatland import (
    Compound,
    DateYYYYMMDD,
    Dict,
    Integer,
    JoinedString,
    String,
    )

from tests._util import assert_raises, eq_, raises


def test_compound_init_sequencing():

    class Canary(Exception):
        pass

    class InitNotCalledAtClassConstruction(Compound):

        @classmethod
        def __compound_init__(cls):
            raise Canary

    # __compound_init_ not called until an instance is constructed
    assert True

    # called when an instance is created
    assert_raises(Canary, InitNotCalledAtClassConstruction)

    # and again, if it fails
    assert_raises(Canary, InitNotCalledAtClassConstruction)


def test_compound_init_sequencing2():
    canary = []

    class MyCompound(Compound):
        field_schema = [String.named(u'x')]

        def __compound_init__(cls):
            assert isinstance(cls, type)
            canary.append(cls)

    # construction of a class will trigger it
    element = MyCompound()
    assert canary == [MyCompound]
    assert MyCompound.__dict__['_compound_prepared']

    # only called once per (known) unique class configuration
    element = MyCompound()
    assert canary == [MyCompound]

    # manual configuration doesn't trigger it
    MyCompound.name = u'set by hand'
    element = MyCompound()
    assert canary == [MyCompound]

    # a clone of the class will trigger it
    MyCompound2 = MyCompound.named(u'something')
    element = MyCompound2()
    assert canary == [MyCompound, MyCompound2]
    assert MyCompound2.__dict__['_compound_prepared']

    # instance-level configuration will also
    element = MyCompound2(name=u'instance')
    assert type(element) is not MyCompound2
    assert isinstance(element, MyCompound2)
    assert canary == [MyCompound, MyCompound2, type(element)]
    assert type(element).__dict__['_compound_prepared']

    del canary[:]

    # it can be run by hand
    MyCompound4 = MyCompound.named(u'somethingelse')
    assert not MyCompound4.__dict__.get('_compound_prepared')

    MyCompound4.__compound_init__()

    assert MyCompound4.__dict__['_compound_prepared']
    assert canary == [MyCompound4]

    # and it won't be re-run automatically
    element = MyCompound4()
    assert canary == [MyCompound4]

    class MyCompound5(Compound):
        field_schema = [String.named(u'x')]

        @classmethod
        def __compound_init__(cls):
            assert isinstance(cls, type)
            canary.append(cls)

    # it can be @classmethod or not
    element = MyCompound5()
    assert canary == [MyCompound4, MyCompound5]


def test_compound_snarfs_override_init_keywords():

    class MyCompound(Compound):
        field_schema = [String.named(u'x')]
        attr = 'value'

        def __init__(self, value='sentinel', abc=1):
            # __new__'s value= should not trump this one
            assert value == 'sentinel'
            self.abc = abc

    mc1 = MyCompound()
    assert mc1.abc == 1

    mc2 = MyCompound(attr='new value')
    assert mc2.abc == 1
    assert mc2.attr == 'new value'
    assert mc1.attr == 'value'
    assert type(mc2) is not type(mc1)

    assert_raises(TypeError, MyCompound, unknown='boom')


def test_compound_metaclass_calls_new():
    canary = []

    class MyCompound(Compound):
        name = u'abc'
        field_schema = [String.named(u'x')]

        def __new__(cls, *args, **kw):
            canary.append('new')
            return Compound.__new__(cls, *args, **kw)

        def __init__(self, *args, **kw):
            canary.append('init')

    assert MyCompound() is not None
    assert canary == ['new', 'init']

    del canary[:]

    class AltNewReturn(Compound):
        name = u'abc'
        field_schema = [String.named(u'x')]

        def __new__(cls, *args, **kw):
            canary.append('new')
            self = {}
            return self

        def __init__(self, *args, **kw):
            canary.append('init')

    assert AltNewReturn() is not None
    assert canary == ['new']


class TestDoubleField(object):

    def setup(self):

        class Double(Compound):

            field_schema = [Integer.named(u'x'), Integer.named(u'y')]

            def compose(self):
                ex, ey = self.get(u'x'), self.get(u'y')
                ux, uy = ex.u, ey.u
                if ex.u and ey.u:
                    string = u"%sx%s" % (ex.u, ey.u)
                else:
                    string = u''

                if ex.value is not None and ey.value is not None:
                    value = (ex.value, ey.value)
                else:
                    value = None

                return string, value

            def explode(self, value):
                if value == u'boom':
                    raise AttributeError('boom')
                if value == u'return-none':
                    return
                try:
                    x, y = value
                except (TypeError, ValueError):
                    return False
                self[u'x'].set(x)
                self[u'y'].set(y)
                return True

        self.Double = Double

    def test_from_flat(self):
        s = self.Double.named(u's')

        e = s.from_flat({})
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        e = s.from_flat({u's': u'1x2'})
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        e = s.from_flat({u's_x': u'1'})
        assert_values_(e, None, 1, None)
        assert_us_(e, u'', u'1', u'')

        e = s.from_flat({u's_x': u'1', u's_y': u'2'})
        assert_values_(e, (1, 2), 1, 2)
        assert_us_(e, u'1x2', u'1', u'2')

        e = s.from_flat({u's_x': u'1', u's_y': u'2', u's': u'5x5'})
        assert_values_(e, (1, 2), 1, 2)
        assert_us_(e, u'1x2', u'1', u'2')

    def test_flatten(self):
        s = self.Double.named(u's')
        e = s(u'1x2')
        eq_(set(e.flatten()),
            set([(u's', u''), (u's_y', u''), (u's_x', u'')]))

    def test_set(self):
        schema = self.Double.named(u's')

        e = schema()
        assert not e.set({})
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        # return-none and boom should get moved to their own test
        e = schema()
        assert e.set(u'return-none')
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        e = schema()
        assert not e.set(u'boom')
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        e = schema()
        assert not e.set(u'4x5')
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

        e = schema()
        assert e.set((4, 5))
        assert_values_(e, (4, 5), 4, 5)
        assert_us_(e, u'4x5', u'4', u'5')

    def test_set_default(self):
        s = self.Double.named(u's').using(default=(4, 5))
        el = s()
        assert_values_(el, None, None, None)
        assert_us_(el, u'', u'', u'')

        el.set_default()
        assert_values_(el, (4, 5), 4, 5)
        assert_us_(el, u'4x5', u'4', u'5')

    def test_set_default_from_children(self):
        schema = self.Double.named(u's')

        fields = dict((field.name, field) for field in schema.field_schema)
        fields[u'x'].default = 4
        fields[u'y'].default = 5

        el = schema()
        assert_values_(el, None, None, None)
        assert_us_(el, u'', u'', u'')

        el.set_default()
        assert_values_(el, (4, 5), 4, 5)
        assert_us_(el, u'4x5', u'4', u'5')

    def test_update(self):
        s = self.Double.named(u's')

        e = s((4, 5))
        assert_values_(e, (4, 5), 4, 5)
        assert_us_(e, u'4x5', u'4', u'5')

        e[u'x'].set(6)
        assert_values_(e, (6, 5), 6, 5)
        assert_us_(e, u'6x5', u'6', u'5')

        e.set((7, 8))
        assert_values_(e, (7, 8), 7, 8)
        assert_us_(e, u'7x8', u'7', u'8')

    def test_explode_does_not_require_typechecks(self):
        s = self.Double.named(u's')

        e = s((4, 5))
        assert_values_(e, (4, 5), 4, 5)
        assert_us_(e, u'4x5', u'4', u'5')

        e.set((7, u'blagga'))
        assert_values_(e, None, 7, None)
        assert_us_(e, u'7xblagga', u'7', u'blagga')

    def test_value_assignement(self):
        s = self.Double.named(u's')

        e = s()
        e.value = (1, 2)
        assert_values_(e, (1, 2), 1, 2)
        assert_us_(e, u'1x2', u'1', u'2')

        e = s()
        e.u = u"1x2"
        assert_values_(e, None, None, None)
        assert_us_(e, u'', u'', u'')

    def test_repr(self):
        s = self.Double.named(u's')
        e = s()
        assert repr(e)

        e.set((1, 2))
        assert repr(e)

        e.set(None)
        assert repr(e)


def test_repr_always_safe():
    # use the abstract class to emulate a subclass with broken compose.
    broken_impl = Compound.using(field_schema=[String.named(u'y')])
    e = broken_impl()
    assert repr(e)


@raises(NotImplementedError)
def test_explode_abstract():
    schema = Compound.using(field_schema=[String.named(u'y')])
    el = schema()
    el.set(u'x')


@raises(NotImplementedError)
def test_compose_abstract():
    schema = Compound.using(field_schema=[String.named(u'y')])
    el = schema()
    el.value


@raises(TypeError)
def test_compose_abstract_fixme():
    # really it'd be nice if serialize simply wasn't inherited. would
    # have to rejigger the hierarchy, not sure its worth it.
    schema = Compound.using(field_schema=[String.named(u'y')])
    el = schema()
    schema.serialize(el, u'abc')


def assert_values_(el, top, x, y):
    eq_(el.value, top)
    eq_(el[u'x'].value, x)
    eq_(el[u'y'].value, y)


def assert_us_(el, top, x, y):
    eq_(el.u, top)
    eq_(el[u'x'].u, x)
    eq_(el[u'y'].u, y)


def test_sample_compound():
    s = DateYYYYMMDD.named(u's')
    s.__compound_init__()

    assert ([field.name for field in s.field_schema] ==
            [u'year', u'month', u'day'])

    e = s()
    assert e.value is None
    assert e[u'year'].value is None
    assert e.el(u'year').value is None

    when = datetime.date(2000, 10, 1)
    e.set(when)
    assert e.value == when
    assert e.el(u'year').value == 2000
    assert e.el(u'day').value == 1
    assert e.u == u'2000-10-01'
    assert e.el(u'day').u == u'01'

    e.el(u'day').set(5)
    assert e.value == datetime.date(2000, 10, 5)
    assert e.el(u'day').value == 5
    assert e.u == u'2000-10-05'

    e = s()
    e.set(None)
    assert e.value is None
    assert e.u == u''
    assert e[u'year'].value is None
    assert e.el(u'year').value is None

    e = s()
    e.set(u'snaggle')
    assert e.value is None
    assert e.u == u''
    assert e[u'year'].value is None
    assert e.el(u'year').value is None


def test_compound_optional():

    required = Dict.of(DateYYYYMMDD.named(u's').using(optional=False))

    f = required.from_defaults()
    assert not f.el(u's.year').optional
    assert not f.el(u's.month').optional
    assert not f.el(u's.day').optional
    assert not f.validate()

    optional = Dict.of(DateYYYYMMDD.named(u's').using(optional=True))

    f = optional.from_defaults()
    assert f.el(u's.year').optional
    assert f.el(u's.month').optional
    assert f.el(u's.day').optional
    assert f.validate()


def test_compound_is_empty():
    element = DateYYYYMMDD()
    assert element.is_empty

    element.el(u'year').set(1979)
    assert not element.is_empty


def test_joined_string():
    el = JoinedString(u'abc')
    assert el.value == u'abc'
    assert [child.value for child in el] == [u'abc']

    el = JoinedString(u'abc,def')
    assert el.value == u'abc,def'
    assert [child.value for child in el] == [u'abc', u'def']

    el = JoinedString([u'abc', u'def'])
    assert el.value == u'abc,def'
    assert [child.value for child in el] == [u'abc', u'def']

    el = JoinedString(u' abc,,ghi ')
    assert el.value == u'abc,ghi'
    assert [child.value for child in el] == [u'abc', u'ghi']

    el = JoinedString(u'abc,,ghi', prune_empty=False)
    assert el.value == u'abc,,ghi'
    assert [child.value for child in el] == [u'abc', u'', u'ghi']

    # The child (String) strips by default
    el = JoinedString(u' abc ,, ghi ', strip=False)
    assert el.value == 'abc,ghi'
    assert [child.value for child in el] == [u'abc', u'ghi']

    # Try with a non-stripping String
    el = JoinedString(u' abc ,, ghi ',
                      strip=False,
                      member_schema=String.using(strip=False))
    assert el.value == ' abc , ghi '
    assert [child.value for child in el] == [u' abc ', u' ghi ']


def test_joined_string_flat():
    schema = JoinedString.named('js').of(Integer)

    for set_value, flat_value in [
        ([1], (u'js', u'1')),
        (u'1', (u'js', u'1')),
        ([1, 2, 3], (u'js', u'1,2,3')),
        (u'1,2,3', (u'js', u'1,2,3'))]:
        from_set = schema(set_value)
        assert from_set.flatten() == [flat_value]
        from_flat = schema.from_flat([flat_value])
        assert len(from_flat) == len(from_set)
        assert from_flat.flatten() == [flat_value]
        assert from_flat.value == from_set.value


def test_joined_string_regex():
    schema = JoinedString.using(separator=u', ',
                                separator_regex=re.compile('X*,X*'))
    el = schema(u'abc')
    assert el.value == u'abc'
    assert [child.value for child in el] == [u'abc']

    el = schema(u'abcX,Xdef')
    assert el.value == u'abc, def'
    assert [child.value for child in el] == [u'abc', u'def']


def test_joined_non_string():
    schema = JoinedString.using(member_schema=Integer)
    el = schema(u'1')
    assert el.value == u'1'
    assert [child.value for child in el] == [1]

    el = schema(u'1, 2, 3')
    assert el.value == u'1,2,3'
    assert [child.value for child in el] == [1, 2, 3]

    el = schema([1, 2, 3])
    assert el.value == u'1,2,3'
    assert [child.value for child in el] == [1, 2, 3]



from flatland import String
from flatland.schema.properties import Properties

from nose.tools import assert_raises


def test_empty():
    class Base(object):
        properties = Properties()

    assert not Base.properties
    b = Base()
    assert not b.properties
    assert not Base.properties

    assert Base.properties == {}
    assert Base().properties == {}

    class Sub(Base):
        pass

    assert Sub.properties == {}
    assert Sub().properties == {}

    Sub().properties['abc'] = 123

    assert Sub.properties == {}
    assert Sub().properties == {}
    assert Base.properties == {}
    assert Base().properties == {}

    Sub.properties['def'] = 456
    assert Base.properties == {}
    assert Base().properties == {}


def test_dictlike():
    class Base(object):
        properties = Properties({'def': 456}, abc=123)

    props = Base.properties
    assert sorted(props.items()) == [('abc', 123), ('def', 456)]

    assert sorted(props.keys()) == ['abc', 'def']
    assert sorted(props.iterkeys()) == ['abc', 'def']

    assert sorted(props.values()) == [123, 456]
    assert sorted(props.itervalues()) == [123, 456]

    assert props.get('abc') == 123
    assert props.get('abc', 'blah') == 123
    assert props.get('blah', 'default') == 'default'
    assert props.get('blah') is None

    assert_raises(NotImplementedError, props.popitem)

    assert 'abc' in props
    assert 'ghi' not in props

    assert props == {'abc': 123, 'def': 456}
    assert props != {'ghi': 789}

    assert props
    props.clear()
    assert not props

    assert repr(props) == '{}'


def test_instance_population():
    class Base(object):
        properties = Properties()

    assert not Base.properties
    b = Base()
    b.properties.update(a=1, b=2, c=3)

    assert b.properties == {'a': 1, 'b': 2, 'c': 3}
    assert Base.properties == {}

    class Sub(Base):
        pass

    assert Sub.properties == {}
    s = Sub()
    assert s.properties == {}

    s.properties['d'] = 4
    assert s.properties == {'d': 4}
    assert Sub.properties == {}
    assert Base.properties == {}
    assert Sub().properties == {}


def test_instance_overlay():
    class Base(object):
        properties = Properties()

    Base.properties['a'] = 1
    b = Base()
    b.properties['b'] = 2

    assert Base.properties == {'a': 1}
    assert b.properties == {'a': 1, 'b': 2}
    del b.properties['a']
    assert b.properties == {'b': 2}
    assert Base.properties == {'a': 1}

    b.properties.update(b='x', c=3, d=4)
    assert b.properties['b'] == 'x'
    assert b.properties == {'b': 'x', 'c': 3, 'd': 4}

    del b.properties['b']
    assert b.properties == {'c': 3, 'd': 4}
    assert_raises(KeyError, lambda: b.properties['b'])

    assert b.properties.setdefault('e', 5) == 5
    assert b.properties.setdefault('e', 'blah') == 5

    assert b.properties == {'c': 3, 'd': 4, 'e': 5}

    assert b.properties.pop('e', 'blah') == 5
    assert b.properties.pop('e', 'blah') == 'blah'
    assert_raises(KeyError, b.properties.pop, 'e')

    b.properties.clear()
    assert b.properties == {}
    assert Base.properties == {'a': 1}

    Base.properties['b'] = 2
    assert b.properties == {'b': 2}
    assert Base.properties == {'a': 1, 'b': 2}

    Base.properties.update(c=3, d=4, e=5)
    del Base.properties['e']
    assert b.properties == {'b': 2, 'c': 3, 'd': 4}
    assert Base.properties == {'a': 1, 'b': 2, 'c': 3, 'd': 4}


def test_instance_member_assignment():

    class Base(object):
        properties = Properties(abc=123)

    b = Base()
    assert b.properties == {'abc': 123}
    b.properties = {'abc': 'detached'}

    assert b.properties == {'abc': 'detached'}

    Base.properties['def'] = 456

    assert b.properties == {'abc': 'detached'}


def test_subclass_overlay():
    class Base(object):
        properties = Properties()

    class Middle(Base):
        pass

    class Lowest(Middle):
        pass

    Lowest.properties['def'] = 456

    assert Base.properties == {}
    assert Middle.properties == {}
    assert Lowest.properties == {'def': 456}

    Base.properties['abc'] = 123

    assert Base.properties == {'abc': 123}
    assert Middle.properties == {'abc': 123}
    assert Lowest.properties == {'abc': 123, 'def': 456}

    del Middle.properties['abc']

    assert Base.properties == {'abc': 123}
    assert 'abc' in Base.properties

    assert Middle.properties == {}
    assert 'abc' not in Middle.properties
    assert_raises(KeyError, lambda: Middle.properties['abc'])

    assert Lowest.properties == {'def': 456}
    assert 'abc' not in Lowest.properties
    assert_raises(KeyError, lambda: Lowest.properties['abc'])

    Middle.properties.setdefault('ghi', 789)
    Middle.properties.setdefault('ghi', 'blah')

    assert Base.properties == {'abc': 123}
    assert Middle.properties == {'ghi': 789}
    assert Lowest.properties == {'ghi': 789, 'def': 456}

    assert Lowest.properties.pop('def', 'blah') == 456
    assert Lowest.properties.pop('def', 'blah') == 'blah'
    assert_raises(KeyError, Lowest.properties.pop, 'def')

    assert Base.properties == {'abc': 123}
    assert Middle.properties == {'ghi': 789}
    assert Lowest.properties == {'ghi': 789}

    Lowest.properties.clear()

    assert Base.properties == {'abc': 123}
    assert Middle.properties == {'ghi': 789}
    assert Lowest.properties == {}


def test_subclass_override():
    class Base(object):
        properties = Properties()

    class Middle(Base):
        pass

    class Override(Middle):
        properties = Properties({'def': 456})

    assert Override.properties == {'def': 456}
    assert Middle.properties == {}
    assert Base.properties == {}

    Base.properties['abc'] = 123

    assert Base.properties == {'abc': 123}
    assert Middle.properties == {'abc': 123}
    assert Override.properties == {'def': 456}


def test_initialization():
    class Base(object):
        properties = Properties(abc=123)

    assert Base.properties == {'abc': 123}
    Base.properties['def'] = 456

    assert Base.properties == {'abc': 123, 'def': 456}
    del Base.properties['abc']
    assert Base.properties == {'def': 456}


def test_perverse():
    class Base(object):
        properties = Properties()

    descriptor = Base.__dict__['properties']
    props = Base.properties
    del Base.properties
    assert list(props._frames()) == []

    def unattached_properties():
        class Unrelated(object):
            pass

        return descriptor.__get__(None, Unrelated)

    lost = unattached_properties()
    assert lost == {}

    lost2 = unattached_properties()
    assert_raises(KeyError, lambda: lost2['abc'])

    class Broken(object):
        properties = 'something else'

    broken = descriptor.__get__(None, Broken)
    broken.update(abc=123)
    assert broken == {'abc': 123}
    assert Broken.properties == 'something else'


def test_perverse_slots():

    class Base(object):
        __slots__ = 'properties',
        properties = Properties()

    b = Base()
    assert_raises(AttributeError, lambda: b.properties['abc'])


def test_dsl():
    Sub = String.with_properties(abc=123)

    assert 'abc' not in String.properties
    assert Sub.properties['abc'] == 123

    Disconnected = Sub.using(properties={'def': 456})
    assert Disconnected.properties['def'] == 456
    assert 'abc' not in Disconnected.properties

    assert 'def' not in Sub.properties
    assert 'def' not in String.properties

    Sub.properties['ghi'] = 789
    assert Disconnected.properties == {'def': 456}

    Disconnected2 = Sub.using(properties=Properties(jkl=123))
    assert Disconnected2.properties == {'jkl': 123}



"""Utilities."""
from base import Maybe, Unspecified, adict, as_mapping,\
        assignable_class_property, assignable_property,\
        autodocument_from_superclasses, class_cloner,\
        keyslice_pairs, lazy_property, luhn10,\
        named_int_factory, re_ucompile, re_uescape, symbol, threading, to_pairs

'''
from .deferred import deferred_module


deferred_module.shadow(
    'flatland.util',
    {
        'base': (
            'Maybe',
            'Unspecified',
            'adict',
            'as_mapping',
            'assignable_class_property',
            'assignable_property',
            'autodocument_from_superclasses',
            'class_cloner',
            'decorator',
            'format_argspec_plus',
            'keyslice_pairs',
            'lazy_property',
            'luhn10',
            'named_int_factory',
            're_ucompile',
            're_uescape',
            'symbol',
            'threading',
            'to_pairs',
            ),
        'deferred': (
            'deferred_module',
            ),
      })
'''



import sys
from types import ModuleType


__all__ = ['deferred_module']


class deferred_module(ModuleType):
    """A module whose __all__ members are loaded on first access."""

    @classmethod
    def shadow(cls, module_name, deferred, **attributes):
        """Replace *module_name* in ``sys.modules`` with a deferred clone."""
        module = sys.modules[module_name]
        sys.modules[module_name] = cls(module, deferred, **attributes)
        print '[deferred_module] shadow()'

    def __init__(self, module, deferred, **attributes):
        print '[deferred_module] __init__()'
        ModuleType.__init__(self, module.__name__, module.__doc__ or None)
        self.__dict__.update(attributes)
        self.__shadowing = module
        self.__all__ = []
        self.__pushed_up = {}
        self.__file__ = module.__file__
        self.__path__ = module.__path__

        for submodule, pushed_up in deferred.iteritems():
            self.__all__.append(submodule)
            if pushed_up:
                for member in pushed_up:
                    self.__pushed_up[member] = submodule
                self.__all__.extend(pushed_up)

    def __getattr__(self, key):
        if key in self.__pushed_up:
            owner = self.__pushed_up[key]
            value = getattr(getattr(self, owner), key)
            setattr(self, key, value)
            return value
        elif key in self.__all__:
            module = __import__(
                self.__name__ + '.' + key, None, None, [self.__name__])
            setattr(self, key, module)
            return module
        else:
            try:
                return ModuleType.__getattribute__(self, key)
            except AttributeError:
                raise AttributeError(
                    'module %r has no attribute %r' % (self.__name__, key))

    def __dir__(self):
        pool = sorted(set(self.__all__).union(self.__dict__.keys()))
        return [key for key in pool if not key.startswith('_deferred_module')]



# -*- coding: utf-8; fill-column: 78 -*-
# TODO: Temporal stripping
import datetime
import decimal
import re

from flatland.exc import AdaptationError
from flatland.util import (
    Unspecified,
    as_mapping,
    autodocument_from_superclasses,
    class_cloner,
    lazy_property,
    )
from flatland.schema.base import Element


__all__ = (
    'Boolean',
    'Date',
    'DateTime',
    'Enum',
    'Float',
    'Integer',
    'Long',
    'Ref',
    'Scalar',
    'String',
    'Time',
    )


class Scalar(Element):
    """The base implementation of simple values such as a string or number.

    Scalar subclasses are responsible for translating the most common data
    types in and out of Python-native form: strings, numbers, dates, times,
    Boolean values, etc.  Any data which can be represented by a single
    ``(name, value)`` pair is a likely Scalar.

    Scalar subclasses have two responsibilities: provide a method to adapt a
    value to native Python form, and provide a method to serialize the native
    form to a Unicode string.

    This class is abstract.

    """
    flattenable = True

    validates_down = 'validators'

    def set(self, value):
        """Assign the native and Unicode value.

        :returns: True if adaptation of *value* was successful.

        Attempts to adapt the given value and assigns this element's
        :attr:`~flatland.Element.value` and :attr:`u`
        attributes in tandem.

        If adaptation succeeds, ``.value`` will contain the
        :meth:`adapted<adapt>` native Python value and ``.u`` will contain a
        Unicode :meth:`serialized<serialize>` version of it.  A native value
        of ``None`` will be represented as ``u''`` in ``.u``.

        If adaptation fails, ``.value`` will be ``None`` and ``.u`` will
        contain ``unicode(value)`` or ``u''`` for none.

        """
        self.raw = value
        try:
            # adapt and normalize the value, if possible
            value = self.value = self.adapt(value)
        except AdaptationError:
            self.value = None
            if value is None:
                self.u = u''
            elif isinstance(value, unicode):
                self.u = value
            else:
                try:
                    self.u = unicode(value)
                except UnicodeDecodeError:
                    self.u = unicode(value, errors='replace')
            return False

        # stringify it, possibly storing what we received verbatim or a
        # normalized version of it.
        if value is None:
            self.u = u''
        else:
            self.u = self.serialize(value)
        return True

    def adapt(self, value):
        """Given any value, try to coerce it into native format.

        :returns: the native format or raises AdaptationError on failure.

        This abstract method is called by :meth:`set`.

        """
        raise NotImplementedError()

    def serialize(self, value):
        """Given any value, coerce it into a Unicode representation.

        :returns: **Must** return a Unicode object, always.

        No special effort is made to coerce values not of native or a
        compatible type.

        This semi-abstract method is called by :meth:`set`.  The base
        implementation returns ``unicode(value)``.

        """
        return unicode(value)

    def _index(self, name):
        raise IndexError(name)

    def _set_flat(self, pairs, sep):
        for key, value in pairs:
            if key == self.name:
                self.set(value)
                break

    def set_default(self):
        default = self.default_value
        if default is not Unspecified:
            self.set(default)

    def __nonzero__(self):
        return True if self.u and self.value else False

    def __unicode__(self):
        return self.u

    def __repr__(self):
        return '<%s %r; value=%r>' % (
            type(self).__name__, self.name, self.value)


    #######################################################################

    def validate_element(self, state, descending):
        """Validates on the first, downward pass.

        See :meth:`FieldSchema.validate_element`.

        """
        # FIXME: now unused
        if descending:
            return FieldSchema.validate_element(
                self, element, state, descending)
        else:
            return None


class String(Scalar):
    """A regular old Unicode string."""

    strip = True
    """If true, strip leading and trailing whitespace during conversion."""

    def adapt(self, value):
        """Return a Unicode representation.

        :returns: a ``unicode`` value or ``None``

        If :attr:`strip` is true, leading and trailing whitespace will be
        removed.

        """
        if value is None:
            return None
        elif self.strip:
            return unicode(value).strip()
        else:
            return unicode(value)

    def serialize(self, value):
        """Return a Unicode representation.

        :returns: a ``unicode`` value or ``u'' if *value* is ``None``

        If :attr:`strip` is true, leading and trailing whitespace will be
        removed.

        """
        if value is None:
            return u''
        elif self.strip:
            return unicode(value).strip()
        else:
            return unicode(value)

    @property
    def is_empty(self):
        """True if the String is missing or has no value."""
        return True if (not self.value and self.u == u'') else False


class Number(Scalar):
    """Base for numeric fields.

    Subclasses provide :attr:`type_` and :attr:`format` attributes for
    :meth:`adapt` and :meth:`serialize`.

    """

    type_ = None
    """The Python type for values, such as ``int`` or ``float``."""

    signed = True
    """If true, allow negative numbers.  Default ``True``."""

    format = u'%s'
    """The ``unicode`` serialization format."""

    def adapt(self, value):
        """Generic numeric coercion.

        :returns: an instance of :attr:`type_` or ``None``

        Attempt to convert *value* using the class's :attr:`type_` callable.

        """
        if value is None:
            return None
        if isinstance(value, basestring):
            value = value.strip()  # decimal.Decimal doesn't like whitespace
        try:
            native = self.type_(value)
        except (ValueError, TypeError, ArithmeticError):
            raise AdaptationError()
        else:
            if not self.signed:
                if native < 0:
                    raise AdaptationError()
            return native

    def serialize(self, value):
        """Generic numeric serialization.

        :returns: a ``unicode`` string formatted with :attr:`format` or the
          ``unicode()`` of *value* if *value* is not of :attr:`type_`

        Converts *value* to a string using Python's string formatting function
        and the :attr:`format` as the template.  The *value* is provided to
        the format as a single, positional format argument.

        """
        if type(value) is self.type_:
            return self.format % value
        return unicode(value)


class Integer(Number):
    """Element type for Python's int."""

    type_ = int
    """``int``"""

    format = u'%i'
    """``u'%i'``"""


class Long(Number):
    """Element type for Python's long."""

    type_ = long
    """``long``"""

    format = u'%i'
    """``u'%i'``"""

class Float(Number):
    """Element type for Python's float."""

    type_ = float
    """``float``"""

    format = u'%f'
    """``u'%f'``"""


class Decimal(Number):
    """Element type for Python's Decimal."""

    type_ = decimal.Decimal
    """``decimal.Decimal``"""

    format = u'%f'
    """``u'%f'``"""


class Boolean(Scalar):
    """Element type for Python's ``bool``."""

    true = u'1'
    """The ``unicode`` serialization for ``True``: ``u'1'``."""

    true_synonyms = (u'on', u'true', u'True', u'1')
    """A sequence of acceptable string equivalents for True.

    Defaults to ``(u'on', u'true', u'True', u'1')``
    """

    false = u''
    """The ``unicode`` serialization for ``False``: ``u''``."""

    false_synonyms = (u'off', u'false', u'False', u'0', u'')
    """A sequence of acceptable string equivalents for False.

    Defaults to ``(u'off', u'false', u'False', u'0', u'')``
    """

    def adapt(self, value):
        """Coerce *value* to ``bool``.

        :returns: a ``bool`` or ``None``

        If *value* is a string, returns ``True`` if the value is in
        :attr:`true_synonyms`, ``False`` if in :attr:`false_synonyms` and
        ``None`` otherwise.

        For non-string values, equivalent to ``bool(value)``.

        """
        if not isinstance(value, basestring):
            return bool(value)
        elif value == self.true or value in self.true_synonyms:
            return True
        elif value == self.false or value in self.false_synonyms:
            return False
        raise AdaptationError()

    def serialize(self, value):
        """Convert ``bool(value)`` to a canonical string representation.

        :returns: either :attr:`self.true` or :attr:`self.false`.

        """
        return self.true if value else self.false


class Constrained(Scalar):
    """A scalar type with a constrained set of legal values.

    Wraps another scalar type and ensures that a value
    :meth:`~flatland.schema.base.Element.set` is within bounds defined by
    :meth:`valid_value`.  If :meth:`valid_value` returns false, the element is
    not converted and will have a :attr:`~flatland.schema.base.Element.value`
    of None.

    :class:`Constrained` is a semi-abstract class that requires an
    implementation of :meth:`valid_value`, either by subclassing or overriding
    on a per-instance basis through the constructor.

    An example of a wrapper of int values that only allows the values of 1, 2
    or 3:

      >>> from flatland import Constrained, Integer
      >>> def is_valid(element, value):
      ...     return value in (1, 2, 3)
      ...
      >>> schema = Constrained.using(child_type=Integer, valid_value=is_valid)
      >>> element = schema()
      >>> element.set(u'2')
      True
      >>> element.value
      2
      >>> element.set(u'5')
      False
      >>> element.value is None
      True

    :class:`Enum` is a subclass which provides a convenient enumerated
    wrapper.

    """

    child_type = String
    """TODO: doc"""

    def __init__(self, value=Unspecified, **kw):
        Scalar.__init__(self, **kw)
        self.child_schema = self.child_type()
        if value is not Unspecified:
            self.set(value)

    @staticmethod
    def valid_value(element, value):
        """Returns True if *value* for *element* is within the constraints.

        This method is abstract.  Override in a subclass or pass a custom
        callable to the :class:`Constrained` constructor.

        """
        return False

    def adapt(self, value):
        value = self.child_schema.adapt(value)
        if not self.valid_value(self, value):
            raise AdaptationError()
        return value

    def serialize(self, value):
        return self.child_schema.serialize(value)

    def validate_element(self, state, descending):
        return self.valid_value(self, state)


class Enum(Constrained):
    """A scalar type with a limited set of allowed values.

    By default values are :class:`strings<String>`, but can be of any type you
    like by customizing :attr:`~Constrained.child_type`.

    """

    @class_cloner
    def valued(cls, *enum_values):
        """Return a class with ``valid_values`` = *enum_values*

        :param \*enum_values: zero or more values for :attr:`valid_values`.
        :returns: a new class

        """
        cls.valid_values = enum_values
        return cls

    valid_values = ()
    """Valid element values.

    Attempting to :meth:`set` a value not present in *valid_values* will cause
    an adaptation failure, and :attr:`value` will be ``None``.

    """

    def __init__(self, value=Unspecified, **kw):
        Constrained.__init__(self, **kw)
        if value is not Unspecified:
            self.set(value)

    def valid_value(self, element, value):
        """True if *value* is within :attr:`valid_values`."""
        return value in self.valid_values

class Temporal(Scalar):
    """Base for datetime-based date and time fields."""

    type_ = None
    regex = None
    format = None
    used = None

    strip = True

    def adapt(self, value):
        """Coerces value to a native type.

        If *value* is an instance of :attr:`type_`, returns it unchanged.  If
        a string, attempts to parse it and construct a :attr:`type` as
        described in the attribute documentation.

        """
        if value is None:
            return value
        elif isinstance(value, self.type_):
            return value
        elif isinstance(value, basestring):
            if self.strip:
                value = value.strip()
            match = self.regex.match(value)
            if not match:
                raise AdaptationError()
            try:
                args = [int(match.group(f)) for f in self.used]
                return self.type_(*args)
            except (TypeError, ValueError):
                raise AdaptationError()
        else:
            raise AdaptationError()

    def serialize(self, value):
        """Serializes value to string.

        If *value* is an instance of :attr:`type`, formats it as described in
        the attribute documentation.  Otherwise returns ``unicode(value)``.

        """
        if isinstance(value, self.type_):
            return self.format % as_mapping(value)
        else:
            return unicode(value)


class DateTime(Temporal):
    """Element type for Python datetime.datetime.

    Serializes to and from YYYY-MM-DD HH:MM:SS format.

    """

    type_ = datetime.datetime
    regex = re.compile(
        ur'^(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2}) '
        ur'(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})$')
    format = (u'%(year)04i-%(month)02i-%(day)02i '
              u'%(hour)02i:%(minute)02i:%(second)02i')
    used = (u'year', u'month', u'day', u'hour', u'minute', u'second')


class Date(Temporal):
    """Element type for Python datetime.date.

    Serializes to and from YYYY-MM-DD format.

    """

    type_ = datetime.date
    regex = re.compile(
        ur'^(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})$')
    format = u'%(year)04i-%(month)02i-%(day)02i'
    used = (u'year', u'month', u'day')


class Time(Temporal):
    """Element type for Python datetime.time.

    Serializes to and from HH:MM:SS format.

    """

    type_ = datetime.time
    regex = re.compile(
        ur'^(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})$')
    format = u'%(hour)02i:%(minute)02i:%(second)02i'
    used = (u'hour', u'minute', u'second')


class Ref(Scalar):
    flattenable = False

    #######################################################################
    writable = 'ignore'

    sep = u'.'

    target_path = None

    @class_cloner
    def to(cls, name, sep=Unspecified):
        if sep is Unspecified:
            sep = cls.sep
        cls.target_path = list(cls._parse_element_path(name, sep))
        if cls.target_path is None:
            raise TypeError("Could not parse path %r" % name)
        return cls

    def adapt(self, value):
        return self.target.adapt(value)

    def serialize(self, value):
        return self.target.serialize(value)

    #######################################################################

    @lazy_property
    def target(self):
        return self.root.el(self.target_path)

    def _get_u(self):
        """The Unicode representation of the reference target."""
        return self.target.u

    def _set_u(self, ustr):
        if self.writable == 'ignore':
            return
        elif self.writable:
            self.target.u = ustr
        else:
            raise TypeError(u'Ref "%s" is not writable.' % self.name)

    u = property(_get_u, _set_u)
    del _get_u, _set_u

    def _get_value(self):
        """The native value representation of the reference target."""
        return self.target.value

    def _set_value(self, value):
        if self.writable == 'ignore':
            return
        elif self.writable:
            self.target.value = value
        else:
            raise TypeError(u'Ref "%s" is not writable.' % self.name)

    value = property(_get_value, _set_value)
    del _get_value, _set_value


for cls_name in __all__:
    autodocument_from_superclasses(globals()[cls_name])
del cls_name



"""Exercise form features.

NOTE: form tests are in the tests.schema package.  This is a legacy
test file, now providing a sample of some possible usage patterns.

"""
from flatland import (
    Dict,
    Form,
    Integer,
    List,
    String,
    )

from tests._util import eq_


REQUEST_DATA = ((u'abc', u'123'),
                (u'surname', u'SN'),
                (u'xjioj', u'1232'),
                (u'age', u'99'),
                (u'fname', u'FN'),
                (u'ns_fname', u'ns_FN'),
                (u'ns_surname', u'ns_SN'),
                (u'ns_snacks_0_name', u'cheez'),
                (u'ns_snacks_1_name', u'chipz'),
                (u'ns_snacks_2_name', u'chimp'),
                (u'ns_squiznart', u'xyyzy'),
                (u'ns_age', u'23'))


class SimpleForm1(Form):
    fname = String
    surname = String
    age = Integer
    snacks = List.of(String.named('name'))


def test_straight_parse():
    form = SimpleForm1.from_flat(REQUEST_DATA)
    eq_(set(form.flatten()), set(((u'fname', u'FN'),
                                  (u'surname', u'SN'),
                                  (u'age', u'99'))))

    eq_(form.value, dict(fname=u'FN',
                         surname=u'SN',
                         age=99,
                         snacks=[]))


def test_namespaced_parse():

    def load(fn):
        form = SimpleForm1.from_defaults(name='ns')
        fn(form)
        return form

    output = dict(fname=u'ns_FN',
                  surname=u'ns_SN',
                  age=23,
                  snacks=[u'cheez', u'chipz', u'chimp'])

    for form in (load(lambda f: f.set_flat(REQUEST_DATA)),
                 load(lambda f: f.set(output))):

        eq_(set(form.flatten()),
            set(((u'ns_fname', u'ns_FN'),
                 (u'ns_surname', u'ns_SN'),
                 (u'ns_age', u'23'),
                 (u'ns_snacks_0_name', u'cheez'),
                 (u'ns_snacks_1_name', u'chipz'),
                 (u'ns_snacks_2_name', u'chimp'))))
        eq_(form.value, output)


def test_default_behavior():

    class SimpleForm2(Form):
        fname = String.using(default=u'FN')
        surname = String

    form = SimpleForm2()
    eq_(form['fname'].value, None)
    eq_(form['surname'].value, None)

    form = SimpleForm2.from_defaults()
    eq_(form['fname'].value, u'FN')
    eq_(form['surname'].value, None)

    class DictForm(Form):
        dict = Dict.of(String.named('fname').using(default=u'FN'),
                       String.named('surname'))

    form = DictForm()
    eq_(form.el('dict.fname').value, None)
    eq_(form.el('dict.surname').value, None)

    form = DictForm.from_defaults()
    eq_(form.el('dict.fname').value, u'FN')
    eq_(form.el('dict.surname').value, None)



from datetime import date
from flatland import (
    Array,
    DateYYYYMMDD,
    Dict,
    Integer,
    Form,
    List,
    )
from flatland.schema.paths import (
    NAME,
    SLICE,
    TOP,
    UP,
    HERE,
    pathexpr,
    tokenize,
    )
from tests._util import assert_raises


def _tokenizes_as(path, expected):
    tokenized = tokenize(path)
    assert tokenized == expected


top = (TOP, None)
up = (UP, None)
here = (HERE, None)
name = lambda x: (NAME, x)
sl = lambda x: (SLICE, x)


def test_tokenize():
    _tokencases = [
        ('.', [here]),
        ('/', [top]),
        ('..', [up]),
        ('/..', [top]),
        ('../..', [up, up]),
        ('//', [top, name(None)]),
        ('foo', [name('foo')]),
        ('/foo', [top, name('foo')]),
        ('foo/', [name('foo')]),
        ('foo/bar', [name('foo'), name('bar')]),
        ('foo/../bar', [name('bar')]),
        ('foo/./bar', [name('foo'), name('bar')]),
        ('foo/./bar/.', [name('foo'), name('bar')]),
        ('foo/bar[bogus]', [name('foo'), name('bar[bogus]')]),
        ('a[b][c:d][0]', [name('a[b][c:d]'), name('0')]),
        ('.[1]', [name('1')]),
        ('foo[1]', [name('foo'), name('1')]),
        ('foo[1]/', [name('foo'), name('1')]),
        ('./foo[1]/', [name('foo'), name('1')]),
        ('foo[1][2][3]', [name('foo'), name('1'), name('2'), name('3')]),
        ('[1][2][3]', [name('1'), name('2'), name('3')]),
        ('[1]/foo/[2]', [name('1'), name('foo'), name('2')]),
        ('[:]', [sl(slice(None))]),
        ('[1:]', [sl(slice(1, None))]),
        ('[1:2]', [sl(slice(1, 2))]),
        ('[:5]', [sl(slice(0, 5))]),
        ('[-5:]', [sl(slice(-5, None))]),
        ('[1:8:2]', [sl(slice(1, 8, 2))]),
        ]
    for path, expected in _tokencases:
        yield _tokenizes_as, path, expected


def test_tokenize_escapes():
    _tokencases = [
        (r'\.', [name('.')]),
        (r'\/', [name('/')]),
        (r'\.\.', [name('..')]),
        (r'/\.\.', [top, name('..')]),
        (r'\/..', [name('/..')]),
        (r'..\/..', [name('../..')]),
        (r'foo\[1]', [name('foo[1]')]),
        (r'foo\/bar', [name(r'foo/bar')]),
        (r'\/foo', [name(r'/foo')]),
        (r'foo\/', [name(r'foo/')]),
        ]
    for path, expected in _tokencases:
        yield _tokenizes_as, path, expected


class Schema(Form):
    i1 = Integer.using(default=0)

    d1 = Dict.of(Integer.named('d1i1').using(default=1),
                 Integer.named('d1i2').using(default=2))

    l1 = List.using(default=2).of(Integer.named('l1i1').using(default=3))

    l2 = List.using(default=3).of(Integer.named('l2i1').using(default=4),
                                  Integer.named('l2i2').using(default=5))

    l3 = List.using(default=2).of(
        List.named('l3l1').using(default=2).of(Integer.using(default=6)))

    a1 = Array.using(default=[10, 11, 12, 13, 14, 15]).of(Integer)

    dt1 = DateYYYYMMDD.using(default=date.today())


def _finds(el, path, expected):
    finder = pathexpr(path)
    elements = finder(el)
    found = [e.value for e in elements]

    if isinstance(expected, set):
        found = set(found)
    assert found == expected


def test_evaluation():
    el = Schema.from_defaults()
    today = date.today()

    _finders = [
        (el['i1'], '.', [0]),
        (el, 'i1', [0]),
        (el, '/i1', [0]),
        (el, '../i1', [0]),
        (el, '../i1/.', [0]),
        (el['i1'], '../i1', [0]),
        (el, 'd1/d1i1', [1]),
        (el, '/d1/d1i2', [2]),
        (el['d1']['d1i1'], '..', [{'d1i1': 1, 'd1i2': 2}]),
        (el, '/l1', [[3, 3]]),
        (el, '/l1[:]', [3, 3]),
        (el, '/l1[2:]', []),
        (el, '/l1[0]', [3]),
        (el, './l1[0]', [3]),
        (el, 'l1/.[0]', [3]),
        (el, '/l2[:]/l2i1', [4, 4, 4]),
        (el, '/l3[:]', [[6, 6], [6, 6]]),
        (el, '/l3[:][:]', [6, 6, 6, 6]),
        (el, 'l3[1:][1:]', [6]),
        (el, 'a1[1:]', [11, 12, 13, 14, 15]),
        (el, 'a1[:-1]', [10, 11, 12, 13, 14]),
        (el, 'a1[3]', [13]),
        (el, 'a1[2]', [12]),
        (el, 'a1[1]', [11]),
        (el, 'a1[0]', [10]),
        (el, 'a1[-1]', [15]),
        (el, 'a1[-2]', [14]),
        (el, 'a1[-3]', [13]),
        (el, 'a1[10]', []),
        (el, 'a1[-10]', []),
        (el, 'a1[-3:-1]', [13, 14]),
        (el, 'a1[1:3]', [11, 12]),
        (el, 'a1[:3]', [10, 11, 12]),
        (el, 'a1[::2]', [10, 12, 14]),
        (el, 'a1[2::2]', [12, 14]),
        (el, 'a1[2:4:2]', [12]),
        (el, 'a1[::-1]', [15, 14, 13, 12, 11, 10]),
        (el, 'a1[::-2]', [15, 13, 11]),
        (el, 'dt1', [today]),
        (el, 'dt1/year', [today.year]),
        (el, 'dt1/./year', [today.year]),
        ]
    for element, path, expected in _finders:
        yield _finds, element, path, expected


def test_find_strict_loose():
    el = Schema.from_defaults()
    _cases = [
        (el, 'bogus'),
        (el, '/d1/d1i3'),
        (el, 'l1[5]'),
        (el, 'l3[:]/missing'),
        ]

    for element, path in _cases:
        assert_raises(LookupError, element.find, path)

        found = element.find(path, strict=False)
        assert not found


def _find_message(el, path, **find_kw):
    try:
        el.find(path, **find_kw)
    except LookupError, exc:
        return str(exc)
    assert False


def test_find_strict_messaging():
    el = Schema.from_defaults()

    message = _find_message(el, 'bogus')
    expected = ("Unnamed element Schema has no child u'bogus' "
                "in expression u'bogus'")
    assert expected in message

    message = _find_message(el, 'd1/bogus')
    expected = ("Dict element u'd1' has no child u'bogus' "
                "in expression u'd1/bogus'")
    assert expected in message

    message = _find_message(el, 'l1[5]')
    expected = ("List element u'l1' has no child u'5' "
                "in expression u'l1[5]'")
    assert expected in message


def test_find_default():
    el = Schema.from_defaults()
    _cases = [
        (el, 'i1', [0]),
        (el, 'a1[:]', [10, 11, 12, 13, 14, 15]),
        (el, 'a1[0]', [10]),
        ]

    for element, path, expected in _cases:
        elements = element.find(path)
        found = [e.value for e in elements]
        assert found == expected


def test_find_single():
    el = Schema.from_defaults()
    _cases = [
        (el, 'i1', 0),
        (el, 'bogus', None),
        (el, 'a1[0]', 10),
        (el, 'a1[10]', None),
        (el, 'l3[1:][1:]', 6),
        ]

    for element, path, expected in _cases:
        element = element.find(path, single=True, strict=False)
        if element is None:
            found = None
        else:
            found = element.value
        assert found == expected


def test_find_single_loose():
    el = Schema.from_defaults()

    element = el.find('l3[:][:]', single=True, strict=False)
    found = element.value
    assert found == 6


def test_find_single_messaging():
    el = Schema.from_defaults()

    message = _find_message(el, 'a1[:]', single=True)
    expected = "Path 'a1[:]' matched multiple elements"
    assert expected in message



from blinker import signal


validator_validated = signal('validator_validated', doc="""\
Emitted after a validator has processed an element.

:param sender: the validator

:param element: the element being validated

:param state: the *state* passed to
  :meth:`~flatland.schema.base.Element.validate`

:param result: the result of validator execution

""")



from functools import wraps

from nose import SkipTest


class Capabilities(dict):

    def __missing__(self, capability):
        probe = getattr(self, '_' + capability)
        self[capability] = have = probe()
        return have

    def _genshi(self):
        try:
            from genshi.template import MarkupTemplate
            # present only in >= 0.6
            return hasattr(MarkupTemplate, 'add_directives')
        except ImportError:
            return False

    def _jinja2(self):
        try:
            import jinja2
        except ImportError:
            return False
        else:
            return True

have = Capabilities()


def need(capability):
    def decorator(fn):
        @wraps(fn)
        def decorated(*args, **kw):
            if not have[capability]:
                raise SkipTest
            return fn(*args, **kw)
        return decorated
    return decorator


def alternate_expectation(backend, string):
    def decorator(fn):
        try:
            alternates = fn.alternates
        except AttributeError:
            alternates = fn.alternates = {}
        alternates[backend] = string.strip()
        return fn
    return decorator


class desired_output(object):

    def __init__(self, language, schema, **kw):
        self.language = language
        self.schema = schema
        self.expected = None
        self.alternate_expectations = {}
        self.render_context = kw

    def __call__(self, fn):
        self.expected = fn.__doc__.strip()
        self.alternate_expectations = getattr(fn, 'alternates', {})
        return self

    def expectation_for(self, backend):
        try:
            return self.alternate_expectations[backend]
        except KeyError:
            return self.expected

    @property
    def genshi(self):
        def decorator(fn):
            markup = _wrap_with_xmlns(fn.__doc__, self.language)
            fn.__doc__ = None
            @wraps(fn)
            def runner():
                if not have['genshi']:
                    raise SkipTest
                got = _render_genshi(markup, self.language, self.schema,
                                     **self.render_context)
                expected = self.expectation_for('genshi')
                if expected != got:
                    print "\n" + fn.__name__
                    print "Expected:\n" + expected
                    print "Got:\n" + got
                assert expected == got
            return runner
        return decorator

    @property
    def markup(self):
        def decorator(fn):
            @wraps(fn)
            def runner():
                got = _render_markup_fn(fn, self.language, self.schema,
                                        **self.render_context)
                expected = self.expectation_for('markup')
                if expected != got:
                    print "\n" + fn.__name__
                    print "Expected:\n" + expected
                    print "Got:\n" + got
                assert expected == got
            return runner
        return decorator


def markup_test(markup='xml', schema=None):
    """Turn a function into a Generator markup test.

    Desired output is read from the docstring.  The function is passed a
    generator and an Element and is expected to return output matching the
    docstring.

    """
    def decorator(fn):
        expected = fn.__doc__.decode('utf8').strip()

        @wraps(fn)
        def test():
            from flatland.out.markup import Generator

            generator = Generator(markup=markup)
            if schema is not None:
                el = schema()
            else:
                el = None
            got = fn(generator, el)
            assert hasattr(got, '__html__')

            got = got.strip()
            if expected != got:
                print "\n" + fn.__name__
                print "Expected:\n" + expected
                print "Got:\n" + got
            assert expected == got
        return test
    return decorator


def render_genshi(markup, language, schema, wrap=True, **context):
    if wrap:
        markup = _wrap_with_xmlns(markup, language)
    return _render_genshi(markup, language, schema, **context)


def _render_markup_fn(fn, language, schema, **kw):
    from flatland.out.markup import Generator

    generator = Generator(markup=language)
    if schema is not None:
        form = schema()
    else:
        form = None
    output = fn(generator, form, **kw)
    return output.strip()


def _render_genshi(markup, language, schema, **kw):
    from genshi.template import MarkupTemplate
    from flatland.out.genshi import setup

    template = MarkupTemplate(markup)
    setup(template)

    if schema is not None:
        kw['form'] = schema()
    else:
        kw['form'] = None
    output = template.generate(**kw).render(language)

    # strip div wrapper off
    got = output[output.index('\n') + 1:output.rindex('\n')]
    got = got.strip()

    return got


def _wrap_with_xmlns(template, language):
    wrapped = '<div '
    if language == 'xhtml':
        wrapped += 'xmlns="http://www.w3.org/1999/xhtml" '
    wrapped += (
        'xmlns:form="http://ns.discorporate.us/flatland/genshi" ' +
        'xmlns:py="http://genshi.edgewall.org/">\n' +
        template +
        '\n</div>')
    return wrapped



from flatland import (
    Dict,
    Integer,
    Ref,
    )

from tests._util import assert_raises


def test_binops():
    schema = Dict.of(Integer.named(u'main'),
                     Ref.named(u'aux').to(u'main'),
                     Integer.named(u'other'))
    el = schema({u'main': 5})

    assert el[u'aux'] == Integer(5)
    assert el[u'aux'].value == 5
    assert el[u'aux'].u == u'5'
    assert Integer(5) == el[u'aux']
    assert 5 == el[u'aux'].value
    assert u'5' == el[u'aux'].u

    assert el[u'aux'] != Integer(6)
    assert el[u'aux'].value != 6
    assert el[u'aux'].u != u'6'
    assert Integer(6) != el[u'aux']
    assert 6 != el[u'aux']
    assert u'6' != el[u'aux']

    assert el[u'aux'] == el[u'main']
    assert el[u'main'] == el[u'aux']
    assert el[u'aux'] != el[u'other']
    assert el[u'other'] != el[u'aux']


def test_writable_ignore():

    def element(writable):
        ref = Ref.named(u'aux').to(u'main')
        if writable:
            ref = ref.using(writable=writable)
        return Dict.of(Integer.named(u'main'), ref)()

    # class default and explicit writable=ignore
    for el in element(None), element('ignore'):
        el[u'aux'] = 6
        assert el[u'main'].value is None
        assert el[u'aux'].value is None


def test_writable():
    schema = Dict.of(Integer.named(u'main'),
                     Ref.named(u'aux').to(u'main').using(writable=True))

    el = schema()
    el[u'aux'] = 6
    assert el[u'main'].value == 6
    assert el[u'aux'].value == 6


def test_not_writable():
    schema = Dict.of(Integer.named(u'main'),
                     Ref.named(u'aux').to(u'main').using(writable=False))

    el = schema()
    assert_raises(TypeError, el[u'aux'].set, 6)


def test_dereference_twice():
    schema = Dict.of(Integer.named(u'main'),
                     Ref.named(u'aux').to(u'main').using(writable=True))

    # Previous revisions would fail after two dereferences
    element = schema()

    element[u'aux'] = 1
    assert element[u'aux'].value == 1

    element[u'aux'].set(1)
    assert element[u'aux'].value == 1

    assert element.el(element[u'aux'].target_path) is element[u'main']
    assert element.el(element[u'aux'].target_path) is element[u'main']



import os
import sys

if sys.version_info < (2, 5):
    print "Python 2.5 or higher is required."
    sys.exit(1)

try:
    from setuptools import setup, find_packages
    extra_setup = dict(
        include_package_data=True,
        zip_safe=True,
        )
except ImportError:
    from distutils.core import setup
    extra_setup = {}
    def find_packages(exclude=()):
        return [w[0].replace('/', '.')
                for w in os.walk('flatland')
                if '__init__.py' in w[2]]

import version
long_desc = open('README').read()

setup(name="flatland-fork",
      version=version.getVersion(),
      packages=find_packages(exclude=['tests.*', 'tests']),
      author='Jason Kirtland',
      author_email='jek@discorporate.us',
      description='HTML form management and validation',
      keywords='schema validation data web form forms roundtrip',
      long_description=long_desc,
      license='MIT License',
      url='http://discorporate.us/jek/projects/flatland/',
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: MIT License',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Programming Language :: Python :: 2.5',
                   'Programming Language :: Python :: 2.6',
                   'Programming Language :: Python :: 2.7',
                   'Topic :: Internet :: WWW/HTTP :: WSGI',
                   'Topic :: Software Development :: Libraries'],
      install_requires=['blinker'],
      **extra_setup)



from weakref import WeakKeyDictionary

from flatland.util import symbol


Deleted = symbol('deleted')


class DictLike(object):

    def iteritems(self):  # pragma: nocover
        raise NotImplementedError

    def items(self):
        return list(self.iteritems())

    def iterkeys(self):
        return (item[0] for item in self.iteritems())

    def keys(self):
        return list(self.iterkeys())

    def itervalues(self):
        return (item[1] for item in self.iteritems())

    def values(self):
        return list(self.itervalues())

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def copy(self):
        return dict(self.iteritems())

    def popitem(self):
        raise NotImplementedError

    def __contains__(self, key):
        return key in self.iterkeys()

    def __nonzero__(self):
        return bool(self.copy())

    def __eq__(self, other):
        return self.copy() == other

    def __ne__(self, other):
        return self.copy() != other

    def __repr__(self):
        return repr(self.copy())


class _TypeLookup(DictLike):
    __slots__ = 'base', 'map', 'descriptor_id'

    def __init__(self, cls, descriptor):
        self.base = cls
        self.map = descriptor.map
        self.descriptor_id = id(descriptor)

    def __getitem__(self, key):
        for frame in self._frames():
            try:
                value = frame[key]
            except KeyError:
                pass
            else:
                if value is Deleted:
                    raise KeyError(key)
                return value
        raise KeyError(key)

    def __setitem__(self, key, value):
        self._base_frame[key] = value

    def __delitem__(self, key):
        self[key]  # must exist to delete
        self._base_frame[key] = Deleted

    def clear(self):
        frame = self._base_frame
        for key in self.iterkeys():
            frame[key] = Deleted

    def pop(self, key, *default):
        try:
            current = self[key]
        except KeyError:
            if not default:
                raise KeyError(key)
            return default[0]
        self[key] = Deleted
        return current

    def setdefault(self, key, default):
        return self._base_frame.setdefault(key, default)

    def update(self, *iterable, **values):
        simplified = dict(*iterable, **values)
        self._base_frame.update(simplified)

    def iteritems(self):
        seen = set()
        for frame in self._frames():
            for key, value in frame.iteritems():
                if key not in seen:
                    seen.add(key)
                    if value is not Deleted:
                        yield (key, value)

    def _frames(self):
        for cls in self.base.__mro__:
            member = cls.__dict__.get('properties')
            if cls not in self.map:
                if member is None or id(member) != self.descriptor_id:
                    continue
                self.map.setdefault(cls, member.initial_set)
            yield self.map[cls]
            if member is not None and id(member) == self.descriptor_id:
                break

    @property
    def _base_frame(self):
        try:
            return self.map[self.base]
        except KeyError:
            pass
        if 'properties' in self.base.__dict__:
            member = self.base.__dict__['properties']
            if id(member) == self.descriptor_id:
                return self.map.setdefault(self.base, member.initial_set)
        return self.map.setdefault(self.base, {})


class local_storage(dict):
    """A marker type for local storage overlays."""


class _InstanceLookup(DictLike):
    __slots__ = 'local', 'class_lookup'

    def __init__(self, instance, class_lookup):
        try:
            local = instance.__dict__.setdefault('properties', local_storage())
        except AttributeError:
            # Descriptor not supported for slots types.
            raise AttributeError(
                "%s object has no attribute 'properties'" % (
                    instance.__class__))
        self.local = local
        self.class_lookup = class_lookup

    def __getitem__(self, key):
        try:
            value = self.local[key]
        except KeyError:
            pass
        else:
            if value is Deleted:
                raise KeyError(key)
            return value
        return self.class_lookup[key]

    def __setitem__(self, key, value):
        self.local[key] = value

    def __delitem__(self, key):
        self[key]  # must exist to delete
        self.local[key] = Deleted

    def clear(self):
        self.local.clear()
        for key in self.class_lookup.keys():
            self.local[key] = Deleted

    def pop(self, key, *default):
        try:
            return self.local.pop(key)
        except KeyError:
            return self.class_lookup.pop(key, *default)

    def setdefault(self, key, default):
        try:
            return self[key]
        except KeyError:
            return self.local.setdefault(key, default)

    def update(self, *iterable, **values):
        simplified = dict(*iterable, **values)
        self.local.update(simplified)

    def iteritems(self):
        seen = set()
        for key, value in self.local.iteritems():
            seen.add(key)
            if value is not Deleted:
                yield key, value
        for key, value in self.class_lookup.iteritems():
            if key not in seen:
                seen.add(key)
                if value is not Deleted:  # pragma: nocover  (coverage bug)
                    yield key, value


class Properties(object):

    def __init__(self, *iterable, **initial_set):
        simplified = dict(*iterable, **initial_set)
        self.initial_set = simplified
        self.map = WeakKeyDictionary()

    def __get__(self, instance, cls):
        class_lookup = _TypeLookup(cls, self)
        if instance is None:
            return class_lookup
        try:
            local = instance.__dict__['properties']
        except (KeyError, AttributeError):
            pass
        else:
            # wholesale assignments to instances replace the inheritance
            # routine entirely
            if type(local) is not local_storage:
                return local
        return _InstanceLookup(instance, class_lookup)

    def __set__(self, instance, value):
        instance.__dict__['properties'] = value



from flatland import (
    Dict,
    Integer,
    List,
    Sequence,
    SkipAll,
    SkipAllFalse,
    String,
    Unevaluated,
    )
from flatland.schema.base import Root
from tests._util import eq_, assert_raises


def test_dsl_of():
    assert_raises(TypeError, Sequence.of)

    t1 = Sequence.of(Integer)
    assert t1.member_schema is Integer

    t2 = Sequence.of(Integer.named(u'x'), Integer.named(u'y'))
    assert issubclass(t2.member_schema, Dict)
    assert sorted(t2.member_schema().keys()) == [u'x', u'y']


def test_dsl_descent_validated_by():
    s = Sequence.using(descent_validators=(123, 456))
    eq_(s.descent_validators, (123, 456))

    s = Sequence.descent_validated_by(123, 456)
    eq_(s.descent_validators, [123, 456])

    s = Sequence.using(descent_validators=(123, 456)).descent_validated_by(789)
    eq_(s.descent_validators, [789])

    assert_raises(TypeError, Sequence.descent_validated_by, int)


def test_dsl_including_descent_validators():
    base = Sequence.descent_validated_by(1, 2, 3)
    eq_(base.descent_validators, [1, 2, 3])

    s = base.including_descent_validators(4, 5, 6)
    eq_(s.descent_validators, [1, 2, 3, 4, 5, 6])

    s = base.including_descent_validators(4, 5, 6, position=0)
    eq_(s.descent_validators, [4, 5, 6, 1, 2, 3])

    s = base.including_descent_validators(4, 5, 6, position=1)
    eq_(s.descent_validators, [1, 4, 5, 6, 2, 3])

    s = base.including_descent_validators(4, 5, 6, position=-2)
    eq_(s.descent_validators, [1, 2, 4, 5, 6, 3])

    s = Sequence.including_descent_validators(1)
    eq_(s.descent_validators, [1])


def test_simple_validation_shortcircuit():
    Regular = Dict.of(Integer.using(optional=False))
    el = Regular()
    assert not el.validate()

    def boom(element, state):
        assert False
    all_ok = lambda element, state: SkipAll

    Boom = Integer.named(u'i').using(validators=[boom])

    ShortCircuited = Dict.of(Boom).using(descent_validators=[all_ok])
    el = ShortCircuited()
    assert el.validate()


class TestContainerValidation(object):

    def setup(self):
        self.canary = []

    def validator(self, name, result):

        def fn(element, state):
            self.canary.append(name)
            return result
        fn.__name__ = name
        return fn

    def test_regular(self):
        schema = (Dict.of(Integer).
                  using(validators=[self.validator('1', True)]))
        el = schema()

        assert not el.validate()
        eq_(self.canary, ['1'])
        assert el.valid
        assert not el.all_valid

    def test_descent(self):
        schema = Dict.of(Integer).using(
            descent_validators=[self.validator('1', True)])
        el = schema()
        assert not el.validate()
        eq_(self.canary, ['1'])
        assert el.valid
        assert not el.all_valid

    def test_paired(self):
        schema = Dict.of(Integer).using(
            descent_validators=[self.validator('1', True)],
            validators=[self.validator('2', True)])
        el = schema()
        assert not el.validate()
        eq_(self.canary, ['1', '2'])
        assert el.valid
        assert not el.all_valid

    def test_paired2(self):
        schema = Dict.of(Integer).using(
            descent_validators=[self.validator('1', False)],
            validators=[self.validator('2', True)])
        el = schema()
        assert not el.validate()
        eq_(self.canary, ['1', '2'])
        assert not el.valid
        assert not el.all_valid

    def test_paired3(self):
        schema = (
            Dict.of(
                Integer.using(validators=[self.validator('2', True)])).
            using(
                descent_validators=[self.validator('1', True)],
                validators=[self.validator('3', True)]))
        el = schema()
        assert el.validate()
        eq_(self.canary, ['1', '2', '3'])
        assert el.valid
        assert el.all_valid

    def test_shortcircuit_down_true(self):
        schema = (
            Dict.of(
                Integer.using(validators=[self.validator('2', False)])).
            using(
                descent_validators=[self.validator('1', SkipAll)],
                validators=[self.validator('3', True)]))
        el = schema()
        assert el.validate()
        eq_(self.canary, ['1', '3'])
        assert el.valid
        assert el.all_valid

    def test_shortcircuit_down_false(self):
        schema = (
            Dict.of(
                Integer.named(u'i').using(
                    validators=[self.validator('2', True)])).
            using(descent_validators=[self.validator('1', SkipAllFalse)],
                  validators=[self.validator('3', True)]))
        el = schema()
        assert not el.validate()
        eq_(self.canary, ['1', '3'])
        assert not el.valid
        assert not el.all_valid
        assert el[u'i'].valid is Unevaluated

    def test_shortcircuit_up(self):
        schema = (
            Dict.of(
                Integer.using(validators=[self.validator('2', True)])).
            using(
                descent_validators=[self.validator('1', True)],
                validators=[self.validator('3', SkipAll)]))
        el = schema()
        assert el.validate()
        eq_(self.canary, ['1', '2', '3'])
        assert el.valid
        assert el.all_valid


def test_sequence():
    schema = Sequence.named(u's')
    assert hasattr(schema, 'member_schema')


def test_mixed_all_children():
    data = {u'A1': u'1',
            u'A2': u'2',
            u'A3': {u'A3B1': {u'A3B1C1': u'1',
                              u'A3B1C2': u'2'},
                    u'A3B2': {u'A3B2C1': u'1'},
                    u'A3B3': [[u'A3B3C0D0', u'A3B3C0D1'],
                             [u'A3B3C1D0', u'A3B3C1D1'],
                             [u'A3B3C2D0', u'A3B3C2D1']]}}

    schema = (
        Dict.named(u'R').of(
            String.named(u'A1'),
            String.named(u'A2'),
            Dict.named(u'A3').of(
                Dict.named(u'A3B1').of(
                    String.named(u'A3B1C1'),
                    String.named(u'A3B1C2')),
                Dict.named(u'A3B2').of(
                    String.named(u'A3B2C1')),
                List.named(u'A3B3').of(
                    List.named(u'A3B3Cx').of(
                        String.named(u'A3B3x'))))))

    top = schema(data)

    names = list(e.name for e in top.all_children)

    assert set(names[0:3]) == set([u'A1', u'A2', u'A3'])
    assert set(names[3:6]) == set([u'A3B1', u'A3B2', u'A3B3'])
    assert set(names[6:12]) == set([u'A3B1C1', u'A3B1C2',
                                    u'A3B2C1', u'A3B3Cx'])
    assert set(names[12:]) == set([u'A3B3x'])
    assert len(names[12:]) == 6


def test_corrupt_all_children():
    # Ensure all_children won't spin out if the graph becomes cyclic.
    schema = List.of(String)
    el = schema()

    el.append(String(u'x'))
    el.append(String(u'y'))
    dupe = String(u'z')
    el.append(dupe)
    el.append(dupe)

    assert list(_.value for _ in el.children) == list(u'xyzz')
    assert list(_.value for _ in el.all_children) == list(u'xyz')


def test_naming_bogus():
    e = String(name=u's')

    assert e.el(u'.') is e
    assert_raises(LookupError, e.el, u'')
    assert_raises(TypeError, e.el, None)
    assert_raises(LookupError, e.el, ())
    assert_raises(LookupError, e.el, iter(()))
    assert_raises(TypeError, e.el)


def test_naming_shallow():
    root = String(name=u's')
    assert root.fq_name() == u'.'
    assert root.flattened_name() == u's'
    assert root.el(u'.') is root

    root = String(name=None)
    assert root.fq_name() == u'.'
    assert root.flattened_name() == u''
    assert root.el(u'.') is root
    assert_raises(LookupError, root.el, ())
    assert root.el([Root]) is root


def test_naming_dict():
    for name, root_flat, leaf_flat in ((u'd', u'd', u'd_s'),
                                       (None, u'', u's')):
        schema = Dict.named(name).of(String.named(u's'))
        root = schema()
        leaf = root[u's']

        assert root.fq_name() == u'.'
        assert root.flattened_name() == root_flat
        assert root.el(u'.') is root

        assert leaf.fq_name() == u'.s'
        assert leaf.flattened_name() == leaf_flat
        assert root.el(u'.s') is leaf
        assert root.el(u's') is leaf
        assert leaf.el(u'.s') is leaf
        assert_raises(LookupError, leaf.el, u's')
        assert leaf.el(u'.') is root

        assert root.el([Root]) is root
        assert root.el([u's']) is leaf
        assert root.el([Root, u's']) is leaf
        assert root.el(iter([u's'])) is leaf
        assert root.el(iter([Root, u's'])) is leaf


def test_naming_dict_dict():
    for name, root_flat, leaf_flat in ((u'd', u'd', u'd_d2_s'),
                                       (None, u'', u'd2_s')):
        schema = Dict.named(name).of(
            Dict.named(u'd2').of(String.named(u's')))
        root = schema()
        leaf = root[u'd2'][u's']

        assert root.fq_name() == u'.'
        assert root.flattened_name() == root_flat
        assert root.el(u'.') is root

        assert leaf.fq_name() == u'.d2.s'
        assert leaf.flattened_name() == leaf_flat
        assert root.el(u'.d2.s') is leaf
        assert root.el(u'd2.s') is leaf
        assert leaf.el(u'.d2.s') is leaf
        assert_raises(LookupError, leaf.el, u'd2.s')
        assert leaf.el(u'.') is root

        assert root.el([u'd2', u's']) is leaf


def test_naming_list():
    for name, root_flat, leaf_flat in ((u'l', u'l', u'l_0_s'),
                                       (None, u'', u'0_s')):
        schema = List.named(name).of(String.named(u's'))
        root = schema([u'x'])
        leaf = root[0]

        assert root.fq_name() == u'.'
        assert root.flattened_name() == root_flat
        assert root.el(u'.') is root

        assert leaf.fq_name() == u'.0'
        assert leaf.flattened_name() == leaf_flat
        assert root.el(u'.0') is leaf
        assert root.el(u'0') is leaf
        assert leaf.el(u'.0') is leaf
        assert_raises(LookupError, leaf.el, u'0')
        assert_raises(LookupError, leaf.el, u's')
        assert leaf.el(u'.') is root

        assert root.el([u'0']) is leaf


def test_naming_list_list():
    # make sure nested Slots-users don't bork
    for name, root_flat, leaf_flat in ((u'l', u'l', u'l_0_l2_0_s'),
                                       (None, u'', u'0_l2_0_s')):
        schema = List.named(name).of(List.named(u'l2').of(String.named(u's')))

        root = schema([u'x'])
        leaf = root[0][0]

        assert root.fq_name() == u'.'
        assert root.flattened_name() == root_flat
        assert root.el(u'.') is root

        assert leaf.fq_name() == u'.0.0'
        assert leaf.flattened_name() == leaf_flat
        assert root.el(u'.0.0') is leaf
        assert root.el(u'0.0') is leaf
        assert leaf.el(u'.0.0') is leaf
        assert_raises(LookupError, leaf.el, u'0')
        assert_raises(LookupError, leaf.el, u's')
        assert leaf.el(u'.') is root

        assert root.el([u'0', u'0']) is leaf



# -*- coding: utf-8; fill-column: 78 -*-
import operator
from ..schema import Slot
from . base import N_, P_, Validator


class NotDuplicated(Validator):
    """A sequence member validator that ensures all sibling values are unique.

    Marks the second and any subsequent occurrences of a value as
    invalid.  Only useful on immediate children of sequence fields
    such as :class:`flatland.List`.

    Example:

    .. testcode::

      import flatland
      from flatland.validation import NotDuplicated

      validator = NotDuplicated(failure="Please enter each color only once.")
      schema = List.of(String.named('favorite_color')).\
                    using(validators=[validator])

    **Attributes**

    .. attribute:: comparator

      A callable boolean predicate, by default ``operator.eq``.
      Called positionally with two arguments, *element* and *sibling*.

      Can be used as a filter, for example ignoring any siblings that
      have been marked as "deleted" by a checkbox in a web form:

      .. testcode::

        from flatland import Form, List, String, Integer, Boolean
        from flatland.validation import NotDuplicated

        def live_addrs(element, sibling):
            thisval, thatval = element.value, sibling.value
            # data marked as deleted is never considered a dupe
            if thisval['deleted'] or thatval['deleted']:
                return False
            # compare elements on 'street' & 'city', ignoring 'id'
            return (thisval['street'] == thatval['street'] and
                    thisval['city'] == thatval['city'])

        class Address(Form):
            validators = [NotDuplicated(comparator=live_addrs)]

            id = Integer.using(optional=True)
            deleted = Boolean
            street = String
            city = String

        schema = List.of(Address)

    .. testcode:: :hide:

        data = {'id': 1, 'deleted': False, 'street': 'a', 'city': 'b'}
        el = schema([data, data])
        assert not el.validate()
        del el[:]
        el.set([data, dict(data, deleted=True)])
        assert el.validate()

    **Messages**

    .. attribute:: failure

      Emitted on an element that has already appeared in a parent
      sequence.  ``container_label`` will substitute the label of the
      container.  ``position`` is the position of the element in the
      parent sequence, counting up from 1.

    """

    # TRANSLATORS: NotDuplicated.failure
    failure = N_(u'%(label)s may not be repeated within %(container_label)s.')

    comparator = operator.eq

    def validate(self, element, state):
        if element.parent is None:
            raise TypeError(
                "%s validator must be applied to a child of a Container "
                "type; %s has no parent." % (
                    type(self).__name__,
                    element.name))
        container = element.parent
        if isinstance(container, Slot):
            container = container.parent
        valid, position = True, 0
        op = self.comparator
        for idx, sibling in enumerate(container.children):
            if sibling is element:
                position = idx + 1
                break
            if valid and op(element, sibling):
                valid = False
        if not valid:
            return self.note_error(
                element, state, 'failure',
                position=position, container_label=container.label)
        return True


class HasAtLeast(Validator):
    """A sequence validator that ensures a minimum number of members.

    May be applied to a sequence type such as a :class:`~flatland.List`.

    Example:

    .. testcode::

      from flatland import List, String
      from flatland.validation import HasAtLeast

      schema = List.of(String.named('wish')).\
                    using(validators=[HasAtLeast(minimum=3)])

    **Attributes**

    .. attribute:: minimum

      Any positive integer.

    **Messages**

    .. attribute:: failure

      Emitted if the sequence contains less than :attr:`minimum` members.
      ``child_label`` will substitute the label of the child schema.

    """

    minimum = 1

    # TRANSLATORS: HasAtLeast.failure
    failure = P_("%(label)s must contain at least one %(child_label)s",
                 "%(label)s must contain at least %(minimum)s "
                 "%(child_label)ss",
                 'minimum')

    def validate(self, element, state):
        assert hasattr(element, 'member_schema'), (
            'container-length validator is only applicable to sequence types.')

        # stupid edge case
        if not self.minimum:
            return True

        if element.value is None or len(element.value) < self.minimum:
            child_label = element.member_schema.label
            return self.note_error(element, state, 'failure',
                                   child_label=child_label)
        return True


class HasAtMost(Validator):
    """A sequence validator that ensures a maximum number of members.

    May be applied to a sequence type such as a :class:`~flatland.List`.

    Example:

    .. testcode::

      from flatland import List, String
      from flatland.validation import HasAtMost

      schema = List.of(String.named('wish')).\
                    using(validators=[HasAtMost(maximum=3)])

    **Attributes**

    .. attribute:: maximum

      Any positive integer.

    **Messages**

    .. attribute:: failure

      Emitted if the sequence contains more than :attr:`maximum` members.
      ``child_label`` will substitute the label of the child schema.

    """

    maximum = 1

    # TRANSLATORS: HasAtMost.failure
    failure = P_("%(label)s must contain at most one %(child_label)s",
                 "%(label)s must contain at most %(maximum)s "
                 "%(child_label)ss",
                 'maximum')

    def validate(self, element, state):
        assert hasattr(element, 'member_schema'), (
            'container-length validator is only applicable to sequence types.')

        if element.value and len(element.value) > self.maximum:
            child_label = element.member_schema.label
            return self.note_error(element, state, 'failure',
                                   child_label=child_label)
        return True


class HasBetween(Validator):
    """Validates that the number of members of a sequence lies within a range.

    May be applied to a sequence type such as a :class:`~flatland.List`.

    Example:

    .. testcode::

      from flatland import List, String
      from flatland.validation import HasBetween

      schema = List.of(String.named('wish')).\
                    using(validators=[HasBetween(minimum=1, maximum=3)])


    **Attributes**

    .. attribute:: minimum

      Any positive integer.

    .. attribute:: maximum

      Any positive integer.

    **Messages**

    .. attribute:: range

      Emitted if the sequence contains fewer than :attr:`minimum` members or
      more than :attr:`maximum` members. ``child_label`` will substitute the
      label of the child schema.

    .. attribute:: exact

      Like :attr:`range`, however this message is emitted if :attr:`minimum`
      and :attr:`maximum` are the same.

      .. testcode::

        schema = List.of(String.named('wish')).\
                      using(validators=[HasBetween(minimum=3, maximum=3)])

    """

    minimum = 1
    maximum = 1

    # TRANSLATORS: HasBetween.range
    range = P_("%(label)s must contain at least %(minimum)s and at most "
               "%(maximum)s %(child_label)s",
               "%(label)s must contain at least %(minimum)s and at most "
               "%(maximum)s %(child_label)ss",
               'maximum')

    # TRANSLATORS: HasBetween.exact
    exact = P_("%(label)s must contain exactly one %(child_label)s",
               "%(label)s must contain exactly %(minimum)s %(child_label)ss",
               'minimum')

    def __init__(self, **kw):
        Validator.__init__(self, **kw)
        if not self.maximum >= self.minimum >= 0:
            raise TypeError("Nonsensical minimum & maximum values (%r, %r)" %
                            (self.minimum, self.maximum))

    def validate(self, element, state):
        assert hasattr(element, 'member_schema'), (
            'container-length validator is only applicable to sequence types.')

        length = len(element.value) if element.value is not None else 0
        if self.minimum <= length <= self.maximum:
            return True

        message = 'exact' if self.minimum == self.maximum else 'range'
        child_label = element.member_schema.label
        return self.note_error(element, state, message,
                               child_label=child_label)



from flatland import (
    Dict,
    String,
    )
from flatland.validation import (
    Converted,
    NoLongerThan,
    )


class GetTextish(object):
    """Mimics a gettext translation."""

    catalog = {
        u'%(label)s is not correct.':
            u'reg %(label)s',

        u'%(label)s may not exceed %(maxlength)s characters.':
            u'default %(label)s %(maxlength)s',

        u'%(label)s max is %(maxlength)s characters.':
            u'plural %(label)s %(maxlength)s',

        u'%(label)s max is %(maxlength)s character.':
            u'single %(label)s %(maxlength)s',

        u'age': u'AGE',

        u'name': u'NAME',
        }

    def ugettext(self, text):
        try:
            return self.catalog[text]
        except KeyError:
            return text

    def ungettext(self, single, plural, n):
        lookup = single if n == 1 else plural
        return self.ugettext(lookup)


class LocalizedShort(NoLongerThan):
    exceeded = (u'%(label)s max is %(maxlength)s character.',
                u'%(label)s max is %(maxlength)s characters.',
                'maxlength')


def test_regular_gettext():
    catalog = GetTextish()

    # translators placed at the top of the form
    schema = Dict.of(String.named('age').using(validators=[Converted()])).\
             using(ugettext=catalog.ugettext,
                   ungettext=catalog.ungettext)

    data = schema()
    data.validate()
    assert data['age'].errors == [u'reg AGE']


def test_local_gettext():
    catalog = GetTextish()

    # translators placed on a specific form element
    schema = Dict.of(String.named('age').using(validators=[Converted()],
                                               ugettext=catalog.ugettext,
                                               ungettext=catalog.ungettext))

    data = schema()
    data.validate()
    assert data['age'].errors == [u'reg AGE']


def test_local_gettext_search_is_not_overeager():
    catalog = GetTextish()

    def poison(*a):
        assert False

    # if a translator is found on an element, its parents won't be searched
    schema = (Dict.of(String.named('age').
                     using(validators=[Converted()],
                           ugettext=catalog.ugettext,
                           ungettext=catalog.ungettext)).
              using(ugettext=poison,
                    ungettext=poison))

    data = schema()
    data.validate()
    assert data['age'].errors == [u'reg AGE']


def test_builtin_gettext():
    catalog = GetTextish()

    schema = Dict.of(String.named('age').using(validators=[Converted()]))

    data = schema()

    try:
        # translators can go into the builtins
        import __builtin__
        __builtin__.ugettext = catalog.ugettext
        __builtin__.ungettext = catalog.ungettext
        data.validate()
        assert data['age'].errors == [u'reg AGE']
    finally:
        del __builtin__.ugettext
        del __builtin__.ungettext


def test_state_gettext():
    catalog = GetTextish()

    schema = Dict.of(String.named('age').using(validators=[Converted()]))

    # if state has ugettext or ungettext attributes, those will be used
    data = schema()
    data.validate(catalog)
    assert data['age'].errors == [u'reg AGE']

    # also works if state is dict-like
    data = schema()
    state = dict(ugettext=catalog.ugettext, ungettext=catalog.ungettext)
    data.validate(state)
    assert data['age'].errors == [u'reg AGE']


def test_tuple_single():
    catalog = GetTextish()
    schema = Dict.of(String.named('name').
                     using(validators=[LocalizedShort(1)]))

    data = schema(dict(name='xxx'))
    data.validate(catalog)
    assert data['name'].errors == [u'single NAME 1']


def test_tuple_plural():
    catalog = GetTextish()
    schema = Dict.of(String.named('name').
                     using(validators=[LocalizedShort(2)]))

    data = schema(dict(name='xxx'))
    data.validate(catalog)
    assert data['name'].errors == [u'plural NAME 2']
